
/**
 * this variable is used for init the id of a new axis. 
*/
var axisIdCounter = 0;

/**
 * The constructor of a new axis
 * axisAttribute: the attribute of the new axis
 * timeIndex: (index * 250ms = time)
*/
function Axis(axisAttribute, timeIndex) {
    updatePlotDimensions();
    this.id = axisIdCounter;
    axisIdCounter++;
    this.attribute = axisAttribute;
    this.time = timeScale[0];
    this.timeIndex = timeScaleIndex[0]; //index im Array
    this.prevAxis = undefined; // reverence to the previous Axis
    this.succAxis = undefined; // reverence to the next Axis
    this.position=0; // position of this axis (horizontal), not pixelspace -> the method rasterize() returns the pixel-coordinate
    this.vanishDrawing = 0; // 0 = no draw, 1 = draw to right, 2 = draw to left;
    this.isOrdinalAxis = isNaN(_DATASET[0][0][axisAttribute]);

    this.axisScale = createAxisScale(axisAttribute, this.time, this.isOrdinalAxis);

    this.axis = d3.svg.axis().scale(this.axisScale).orient('left');
    this.axisElement = undefined; // the svg-Element of this axis
    this.axisElement = svgAxesDescription.append('g').call(this.axis);
    this.axisElement.attr({
        'class': 'axis',
        'axesId': this.id,
        'transform': 'translate(' + rasterize(this.position) + ',0)'
    });
    this.rect = undefined;

    this.brush= d3.svg.brush().y(this.axisScale).on("brushend", brush);
    this.brushElement = this.axisElement.append("g")
        .attr("class","brush")
        .call(this.brush);

    this.brushElement.selectAll("rect")
        .attr("x", -10)
        .attr("width", 16);

    this.axisElement.append("text")
        .attr("y", 25)
        .attr("class","axisLabel")
        .text(this.attribute);

    this.textTime=this.axisElement.append("text")
        .attr("y", parallelPlotHeight - 10)
        .attr("class","axisTime")
        .text("t="+this.time+" ms");
}

/**
 * This method updates all depentend attributes of the axis for a new timeIndex
 * newTimeIndex: the new timeindex for the "axis"
 * axis: axis object
*/
function updateTimeAxis(axis, newTimeIndex) {
    axis.timeIndex = clamp(newTimeIndex, timeScaleIndex[0], timeScaleIndex[1]);
    axis.time = calcTime(axis.timeIndex);
    if(axis.isOrdinalAxis) {
        var typenRange = [];
        for ( var i = 0; i < axis.axisScale.domain().length; i++) {
            typenRange.push((margin + perspectiveScale(axis.time) + i/(axis.axisScale.domain().length-1) * (parallelPlotHeight-2*perspectiveScale(axis.time) - 2*margin)));
        }
        axis.axisScale.range(typenRange);
    } else {
        axis.axisScale.range([parallelPlotHeight-perspectiveScale(axis.time) - margin, margin + perspectiveScale(axis.time)]);
    }

    axis.axisElement.call(axis.axis);

    axis.textTime.text("t="+ axis.time + "ms");

    //Fang for vanishing moving
    if(axis.prevAxis != undefined && axis.prevAxis.attribute === axis.attribute && axis.prevAxis.vanishDrawing ==1) {
        var timeScaleX = d3.scale.linear()
            .domain([axis.prevAxis.time, timeScale[1]])
            .range([rasterize(axis.prevAxis.position), rasterize(axis.prevAxis.position) + (axis.prevAxis.vanishDrawing == 1 ? vanishingPointPixelSize : (-vanishingPointPixelSize))]);
        updateAxisPositionWithoutZoom(axis, timeScaleX(axis.time));
    }
    else if(axis.succAxis != undefined && axis.succAxis.attribute === axis.attribute && axis.succAxis.vanishDrawing ==2) {
        var timeScaleX = d3.scale.linear()
            .domain([axis.succAxis.time, timeScale[1]])
            .range([rasterize(axis.succAxis.position), rasterize(axis.succAxis.position) + (axis.succAxis.vanishDrawing == 1 ? vanishingPointPixelSize : (-vanishingPointPixelSize))]);
        updateAxisPositionWithoutZoom(axis, timeScaleX(axis.time));
    }

}

/**
 * This method updates the postin of the axisElement of the axis
 * axis: this axis will be updateted
 * pos: new position of the axis (not pixel-space)
*/
function updateAxisPosition(axis, pos) {
    axis.position = pos;
    axis.axisElement.attr({
        'transform': 'translate(' + rasterize(axis.position) + ',0)'
    });
    axis.textTime.attr("y", parallelPlotHeight - 10);
}

/**
 * This method is used for setting new position without the shift and zoom attributes.
 * It is needed, when the axis is moved by a move-Event.
*/
function updateAxisPositionWithoutZoom(axis, pos) {
    axis.position= (pos-shift)/zoom;
    axis.axisElement.attr({
        'transform': 'translate(' + pos + ',0)'
    });
}

/**
 * This method renders an axis.
 * It updates all dependent elements like: rects, ticklines, axisLines, timelines, curvedlines
 * It uses the filter object for coloring the lines correct.
 * axis: this axis will be rendered
*/
function renderAxis(axis) {

    //render vanishing poing:
    {
        if(axis.vanishDrawing != 0) {
            var vanishingPoingGroup = d3.select("#vanish-" + axis.id);
            if(vanishingPoingGroup.empty()) {
                vanishingPoingGroup = svgVanishing.append("g");
                vanishingPoingGroup.attr("id", "vanish-" + axis.id);
            }

            var lineGroups = vanishingPoingGroup.selectAll("g").data(filters.getFilterOrderIds());

            lineGroups.enter().append("g");

            lineGroups.attr("id", function(d, i) {
                return "vanish-linegroup-" + axis.id + "-" + i;
            });

            for(var i = 0; i < filters.getFilterOrderIds().length; i++) {
                var tempFilter = filters.getFilterById(filters.getFilterOrderIds()[i]);

                var filteredDataset = [];

                var timeScaleX = d3.scale.linear()
                    .domain([axis.time, timeScale[1]])
                    .range([rasterize(axis.position), rasterize(axis.position) + (axis.vanishDrawing == 1 ? vanishingPointPixelSize : (-vanishingPointPixelSize))]);

                for(var j = 0; j < tempFilter.lines.length; j++) {
                    var tempDataset = tempFilter.lines[j];
                    filteredDataset.push(tempFilter.lines[j].slice(axis.timeIndex, timeScaleIndex[1]+1));
                }

                var lines = d3.select("#vanish-linegroup-" + axis.id + "-" + i).selectAll("path").data(filteredDataset);

                lines.enter().append("path");

                var lineDomain = undefined;
                if(!axis.isOrdinalAxis) {
                    lineDomain = axis.axisScale.domain();
                } else {
                    lineDomain = [Math.min.apply(null, axis.axisScale.range()), Math.max.apply(null, axis.axisScale.range())];
                }

                var line = d3.svg.line()
                    .x(function(d) {
                        return Math.round(timeScaleX(d.time));
                    })
                    .y(function(d) {
                        var range = [parallelPlotHeight-perspectiveScale(d.time) - margin, margin + perspectiveScale(d.time)];
                        if(!axis.isOrdinalAxis) {
                            var domainPercent = (d[axis.attribute] - lineDomain[0]) / (lineDomain[1] - lineDomain[0]);
                        } else {
                            var domainPercent = (axis.axisScale(d[axis.attribute]) - lineDomain[0]) / (lineDomain[1] - lineDomain[0]);
                        }
                        return range[0] + domainPercent * (range[1] - range[0]);
                    }).interpolate("basis");

                lines.attr("d",line);

                lines.attr("class", function(d) {
                    return "linehover line-" + d[0].index;
                });

                lines.attr("style", function(d, i) {
                    var stroke="stroke: rgba(0,0,0, 0.0);";
                    var tempColor = tempFilter.color;
                    color="rgba(" + parseInt(tempColor.substring(1,3),16)+','+parseInt(tempColor.substring(3,5),16)+','+parseInt(tempColor.substring(5,7),16)+','+tempFilter.opacity / 100 +')';

                    stroke = "stroke: " + color;
                    return stroke;
                });

                lines.exit().remove();

            }

            //draw Tick-Lines
            {
                var ticks = [];
                axis.axisElement.selectAll(".tick").each(function(data) {
                    ticks.push(new Tick(data, 0));
                });

                var temptickLines = vanishingPoingGroup.selectAll("line").data(ticks);
                temptickLines.enter().append("line");

                var vanishingScale = d3.scale.linear()
                    .domain(axis.axisScale.domain())
                    .range([parallelPlotHeight-perspectiveScale(axis.time), perspectiveScale(this.time)]);

                if(axis.isOrdinalAxis) {
                    vanishingScale = d3.scale.ordinal().domain(axis.axisScale.domain());
                    var typenRange = [];
                    for ( var j = 0; j < axis.axisScale.domain().length; j++) {
                        typenRange.push((margin + perspectiveScale(timeScale[1]) + j/(axis.axisScale.domain().length-1) * (parallelPlotHeight - 2 * margin -2*perspectiveScale(timeScale[1]))));
                    }
                    vanishingScale.range(typenRange);
                } else {
                    vanishingScale = d3.scale.linear().domain(axis.axisScale.domain())
                        .range([parallelPlotHeight-perspectiveScale(timeScale[1]) - margin, margin + perspectiveScale(timeScale[1])]);
                }

                temptickLines.attr({
                    x1: function(d) { return rasterize(axis.position)},
                    x2: function(d) { return rasterize(axis.position) + (axis.vanishDrawing == 1 ? vanishingPointPixelSize : (-vanishingPointPixelSize))},
                    y1: function(d) { return axis.axisScale(d.data)},
                    y2: function(d) { return vanishingScale(d.data)},
                    class: "gridLines"
                });
                temptickLines.exit().remove();
            }

            //render rects:
            {
                var selectedRect = vanishingPoingGroup.selectAll("polygon");
                if(selectedRect.empty()) {
                    selectedRect = vanishingPoingGroup.append("polygon");
                }
                var x1 = rasterize(axis.position);
                var x2 = rasterize(axis.position) + (axis.vanishDrawing == 1 ? vanishingPointPixelSize : (-vanishingPointPixelSize));
                var y1min = Math.min.apply(null, axis.axisScale.range());
                var y1max = Math.max.apply(null, axis.axisScale.range());
                var y2min = Math.min.apply(null, vanishingScale.range());
                var y2max = Math.max.apply(null, vanishingScale.range());

                selectedRect.attr("points", x1 + "," + y1min + " " + x2 + "," + y2min + " " + x2 + "," + y2max + " " + x1 + "," + y1max);
            }
        } else {
            var vanishingDeletion = document.getElementById("vanish-" + axis.id);
            while (vanishingDeletion != null && vanishingDeletion.firstChild) {
                vanishingDeletion.removeChild(vanishingDeletion.firstChild);
            }
        }
        //updateLinehover();
    }

    try {
        if(axis.prevAxis==undefined) return;
    } catch(ex) {
        return;
    }

    if(axis.lines == undefined) {
        axis.lines = svglines.append('g');
        axis.lines.attr("id", "line-" + axis.id);

        axis.rect = svgRects.append('polygon');
        axis.rect.attr("axesId", axis.id);
        axis.rect.attr("class","hasmenu");
        //axis.rect.call(dragTimeArea);

        axis.tickLines = svgAxesDescription.append('g');
        axis.tickLines.attr("ticklinesId", axis.id);
    }

    if(axis.prevAxis.attribute === axis.attribute) {

        var lineGroups = axis.lines.selectAll("g").data(filters.getFilterOrderIds());

        lineGroups.enter().append("g");

        lineGroups.attr("id", function(d, i) {
            return "linegroup-" + axis.id + "-" + i;
        });

        for(var i = 0; i < filters.getFilterOrderIds().length; i++) {
            var tempFilter = filters.getFilterById(filters.getFilterOrderIds()[i]);

            var filteredDataset = [];

            var tTimeMin = axis.prevAxis.time;
            var tTimeMax = axis.time;

            var timeScaleX = d3.scale.linear()
                .domain([tTimeMin, tTimeMax])
                .range([rasterize(axis.prevAxis.position), rasterize(axis.position)]);

            if(tTimeMin == tTimeMax) {
                timeScaleX.domain([tTimeMin, tTimeMin + timeInterval]);
            }

            for(var j = 0; j < tempFilter.lines.length; j++) {
                var tempDataset = tempFilter.lines[j];
                var tMin = Math.min(axis.prevAxis.timeIndex, axis.timeIndex);
                var tMax = Math.max(axis.prevAxis.timeIndex, axis.timeIndex);
                if(tMin==tMax){
                    var tempArray=[];
                    var tempObject = tempFilter.lines[j][tMin];
                    var copiedObject = jQuery.extend(true, {}, tempObject)
                    copiedObject.time = tempObject.time + timeInterval;
                    tempArray.push(tempObject);
                    tempArray.push(copiedObject);
                    filteredDataset.push(tempArray);
                } else
                    filteredDataset.push(tempFilter.lines[j].slice(tMin,tMax+1));
            }

            var lines = d3.select("#linegroup-" + axis.id + "-" + i).selectAll("path").data(filteredDataset);

            lines.enter().append("path");

            var lineDomain = undefined;
            if(!axis.isOrdinalAxis) {
                lineDomain = axis.axisScale.domain();
            } else {
                lineDomain = [Math.min.apply(null, axis.axisScale.range()), Math.max.apply(null, axis.axisScale.range())];
            }

            var line = d3.svg.line()
                .x(function(d) {
                    return Math.round(timeScaleX(d.time));
                })
                .y(function(d) {
                    var range = [parallelPlotHeight-perspectiveScale(d.time) - margin, margin + perspectiveScale(d.time)];
                    if(!axis.isOrdinalAxis) {
                        var domainPercent = (d[axis.attribute] - lineDomain[0]) / (lineDomain[1] - lineDomain[0]);
                    } else {
                        var domainPercent = (axis.axisScale(d[axis.attribute]) - lineDomain[0]) / (lineDomain[1] - lineDomain[0]);
                    }
                    return range[0] + domainPercent * (range[1] - range[0]);
                }).interpolate("basis");

            lines.attr("d",line);

            lines.attr("class", function(d) {
                return "linehover line-" + d[0].index;
            });

            lines.attr("style", function(d, i) {
                var stroke="stroke: rgba(0,0,0, 0.0);";
                var tempColor = tempFilter.color;
                color="rgba(" + parseInt(tempColor.substring(1,3),16)+','+parseInt(tempColor.substring(3,5),16)+','+parseInt(tempColor.substring(5,7),16)+','+tempFilter.opacity / 100 +')';

                stroke = "stroke: " + color;
                return stroke;
            });

            lines.exit().remove();

        }

        //draw Tick-Lines
        {
            var ticks = [];
            axis.axisElement.selectAll(".tick").each(function(data) {
                var tick = d3.select(this);
                var transform = d3.transform(tick.attr("transform")).translate;
                ticks.push(new Tick(data, transform[1]));
            });

            axis.prevAxis.axisElement.selectAll(".tick").each(function(data, i) {
                var tick = d3.select(this);
                var transform = d3.transform(tick.attr("transform")).translate;
                ticks[i].yP=transform[1];
            });

            var temptickLines = axis.tickLines.selectAll("line").data(ticks);
            temptickLines.enter().append("line");

            temptickLines.attr({
                x1: function(d) { return rasterize(axis.prevAxis.position)},
                x2: function(d) { return rasterize(axis.position)},
                y1: function(d) { return d.yP},
                y2: function(d) { return d.y},
                class: "gridLines"
            });
            temptickLines.exit().remove();
        }
    }
    else
    {
        //calculate values for drawing lines:
        {
            var smothTimeDiffC1 = axis.prevAxis.time + curveSmoothness * (axis.time - axis.prevAxis.time);
            var smothTimeDiffC2 = axis.time - curveSmoothness * (axis.time - axis.prevAxis.time);
            var tempAxixC1Scale = undefined;
            var tempAxixC2Scale = undefined;

            if(axis.prevAxis.isOrdinalAxis) {
                var typenRange = [];
                for ( var i = 0; i < axis.prevAxis.axisScale.domain().length; i++) {
                    typenRange.push((margin + perspectiveScale(smothTimeDiffC1) + i/(axis.prevAxis.axisScale.domain().length-1) * (parallelPlotHeight-2*perspectiveScale(smothTimeDiffC1) - 2*margin)));
                }
                tempAxixC1Scale = d3.scale.ordinal().domain(axis.prevAxis.axisScale.domain()).range(typenRange);
            } else {
                tempAxixC1Scale = d3.scale.linear().domain(axis.prevAxis.axisScale.domain()).range([parallelPlotHeight-perspectiveScale(smothTimeDiffC1) - margin, margin + perspectiveScale(smothTimeDiffC1)]);
            }

            if(axis.isOrdinalAxis) {
                var typenRange = [];
                for ( var i = 0; i < axis.axisScale.domain().length; i++) {
                    typenRange.push((margin + perspectiveScale(smothTimeDiffC2) + i/(axis.axisScale.domain().length-1) * (parallelPlotHeight-2*perspectiveScale(smothTimeDiffC2) - 2*margin)));
                }
                tempAxixC2Scale = d3.scale.ordinal().domain(axis.axisScale.domain()).range(typenRange);
            } else {
                tempAxixC2Scale = d3.scale.linear().domain(axis.axisScale.domain()).range([parallelPlotHeight-perspectiveScale(smothTimeDiffC2) - margin, margin + perspectiveScale(smothTimeDiffC2)]);
            }
        }

        var lineGroups = axis.lines.selectAll("g").data(filters.getFilterOrderIds());

        lineGroups.enter().append("g");

        lineGroups.attr("id", function(d, i) {
            return "linegroup-" + axis.id + "-" + i;
        });

        for(var i = 0; i < filters.getFilterOrderIds().length; i++) {
            var tempFilter = filters.getFilterById(filters.getFilterOrderIds()[i]);
            var lines = d3.select("#linegroup-" + axis.id + "-" + i).selectAll("path").data(tempFilter.lines);

            lines.enter().append("path");

            lines.attr("d", function (d,i){
                var mx=rasterize(axis.prevAxis.position);
                var my=Math.round(axis.prevAxis.axisScale(d[axis.prevAxis.timeIndex][axis.prevAxis.attribute]));
                var cx1=Math.round((rasterize(axis.prevAxis.position) + curveSmoothness * (rasterize(axis.position) - rasterize(axis.prevAxis.position))));
                var cy1=Math.round(tempAxixC1Scale(d[axis.prevAxis.timeIndex][axis.prevAxis.attribute]));
                var cx2=Math.round(rasterize(axis.position) - curveSmoothness*(rasterize(axis.position) - rasterize(axis.prevAxis.position)));
                var cy2=Math.round(tempAxixC2Scale(d[axis.timeIndex][axis.attribute]));
                var cx=rasterize(axis.position);
                var cy=Math.round(axis.axisScale(d[axis.timeIndex][axis.attribute]));

                return 'M ' + mx + ',' + my +
                    ' C ' + cx1 + ',' + cy1 +
                    ' ' + cx2 +',' + cy2 +
                    ' ' + cx +',' + cy;;
            });

            lines.attr("class", function(d) {
                return "linehover line-" + d[axis.timeIndex].index;
            });

            lines.attr("style", function(d, i) {
                var stroke="stroke: rgba(0,0,0, 0.0);";
                var tempColor = tempFilter.color;
                color="rgba(" + parseInt(tempColor.substring(1,3),16)+','+parseInt(tempColor.substring(3,5),16)+','+parseInt(tempColor.substring(5,7),16)+','+tempFilter.opacity / 100 +')';

                stroke = "stroke: " + color;
                return stroke;
            });

            lines.exit().remove();
        }

        lineGroups.exit().remove();
    }

    //render rects:
    {
        var x1 = Math.min(rasterize(axis.prevAxis.position), rasterize(axis.position));
        var x2 = Math.max(rasterize(axis.prevAxis.position), rasterize(axis.position));
        var y1min = Math.min(axis.prevAxis.axisScale.range()[0], axis.prevAxis.axisScale.range()[axis.prevAxis.axisScale.range().length-1]);
        var y1max = Math.max(axis.prevAxis.axisScale.range()[0], axis.prevAxis.axisScale.range()[axis.prevAxis.axisScale.range().length-1]);
        var y2min = Math.min(axis.axisScale.range()[0], axis.axisScale.range()[axis.axisScale.range().length-1]);
        var y2max = Math.max(axis.axisScale.range()[0], axis.axisScale.range()[axis.axisScale.range().length-1]);

        axis.rect.attr("points", x1 + "," + y1min + " " + x2 + "," + y2min + " " + x2 + "," + y2max + " " + x1 + "," + y1max);
    }
    updateLinehover();

}
