/**
 * counter for init id of filters
*/
var filterCounter = 0;
/**
 * the id of the selected filter (blue colored box)
*/
var selectedFilterId = 0;
/**
 * this list stores all filters 
*/
var filterList = [];

var initColor = "#696969";
var initOpacity = 35;
var callbackMethod = undefined;
/**
 * The filterorder is used to draw the lines in the correct order (like the order of the boxes)
*/
var filterOrder = undefined;

/**
 * Constructor of the filters object. It stores the whole information about the filters.
 * initCallback: this method will be called, whenever the filters object is changed. (renderallAxis())
*/
function Filters(initCallback) {
    callbackMethod = initCallback;
    this.getFilterList = function() {
        return filterList;
    }
    this.getFilterOrderIds = function() {
        updateOrder();
        return filterOrder;
    }
    this.updateFilterLines = updateFilterLines;
    this.getFilterById = getFilterById;
    this.setExtentForAxis = function(axis, extent, isExtentEmpty) {
        var axisExists = false;
        var tempFilter = getFilterById(selectedFilterId);
        for(var i = 0; i < tempFilter.filterAxisList.length; i++) {
            if(tempFilter.filterAxisList[i].axis == axis) {
                tempFilter.filterAxisList[i].extent = isExtentEmpty ? [Number.NEGATIVE_INFINITY, Number.POSITIVE_INFINITY] : extent;
                axisExists = true;
            }
        }
        if(!axisExists) {
            tempFilter.filterAxisList.push(new FilterAxis(axis, extent));
        }
    }
    addFilter(undefined);
    initSelectedFilter();
}

/**
 * This method returns the filter with the given id
 * id: searching id (if this id does not exists, the first filter ist returned)
*/
function getFilterById(id){
    for(var i = 0; i < filterList.length; i++) {
        if(filterList[i].id.toString() === id.toString()) return filterList[i];
    }
    return filterList[0]; // return default if no id is found
}

/**
 * Constructor of a Filter object
 * A filter has a color, an opacity and the lines, which are coled with this filter
*/
function Filter() {
    this.id = filterCounter;
    filterCounter++;
    this.color = initColor;
    this.opacity = initOpacity;
    this.filterAxisList = [];
    this.lines = []; //this will be updated, if the filter changes with the method updateFilterLines()
}

/**
 *  This object is used to store the brushing information for an axis (extent)
*/
function FilterAxis(axis, extent) {
    this.axis = axis;
    this.extent = extent;
}

/**
 * adds the class for an init box with the blue color
*/
var initSelectedFilter = function() {
    $("#0").addClass("selected-filterbox");
}

var colorsForPallette = ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#f781bf', 'grey'];
var colorPickerCounter = 0;

/**
 * This method adds a new Filter object. 
 * the colorpicker and sliders are initialized
*/
var addFilter = function(filter) {
    var tempFilter;
    if(filter === undefined){
        tempFilter=new Filter();
    }else{
        tempFilter=filter;
    }
    filterList.push(tempFilter);

    var filterBoxString = "<div id='" + tempFilter.id + "' class='filterbox'><div class='element'><input type='button' value='' style='width: 1.5em; margin-right: 3em;'> <div class='flexgrowing'>Selected " + tempFilter.id + "</div></div><div class='element mt05'><div class='cp-wrapper'><input id='cp-" + tempFilter.id + "' type='text' hidden='true' style='width: 0px;'></div><div id='slider-" + tempFilter.id + "' class='opacity-slider'><div id='slider-value-" + tempFilter.id + "' class='ui-slider-handle'></div></div></div></div>";

    $("#filters").append(filterBoxString);

    $('#cp-' + tempFilter.id).colorpicker({
        showOn: "button",
        color: colorsForPallette[colorPickerCounter % (colorsForPallette.length-1)],
        customTheme: ['#e41a1c','#377eb8','#4daf4a','#984ea3','#ff7f00','#ffff33','#f781bf', 'grey']
    });
    colorPickerCounter++;

    $("#cp-" + tempFilter.id).on("change.color", function(event, color){
        getFilterById(this.getAttribute('id').substr(3)).color=color;
        setTimeout(function() {
            updateFilterLines();
            callbackMethod();
        }, 200);
        //callbackMethod();
    });

    var handle = $( "#slider-value-" + tempFilter.id );
    $( "#slider-" + tempFilter.id ).slider({
        create: function() {
            handle.text(tempFilter.opacity);
        },
        slide: function( event, ui ) {
            handle.text( ui.value );
            getFilterById(this.getAttribute('id').substr(7)).opacity=ui.value;
            updateFilterLines();
            callbackMethod();
        }
    });

    $('#cp-' + tempFilter.id).colorpicker("val", tempFilter.color);
    $( "#slider-" + tempFilter.id).slider( "value", tempFilter.opacity);

    $('#' + tempFilter.id).click(function() { updateSelectedFilterId(this.getAttribute('id')) });
    updateSelectedFilterId(tempFilter.id);
}

/**
 * If the filter is clicked (box) than this method is called.
 * The filter with the "id" is now the active filter (for brushing)
*/
function updateSelectedFilterId(id) {
    selectedFilterId = id;
    for(var i = 0; i < $('.filterbox').length; i++) {
        $('.filterbox').removeClass("selected-filterbox");
    }
    $('#' + selectedFilterId).addClass("selected-filterbox");
    setAllBrushesToSelectedFilter(selectedFilterId);
    onClickSelectTool();
}

/**
 * event handlers for the 3 buttons on left bottom 
*/
var addButton = $('#add-button').click(function() {addFilter(undefined)});

var removeButton = $('#remove-button').click(function() {
    if(filterList.length == 1) return;
    $('#' + selectedFilterId).remove();
    var removeFilter= filterList.indexOf(getFilterById(selectedFilterId));
    filterList.splice(removeFilter,1);
    updateSelectedFilterId(filterList[filterList.length-1].id);
    updateFilterLines();
    callbackMethod();
});

var copyButton = $('#copy-button').click(function() {
    var tempFilter = new Filter();
    var selFilter= getFilterById(selectedFilterId);
    tempFilter.color=selFilter.color;
    tempFilter.opacity=selFilter.opacity;
    for(var i=0;i<selFilter.filterAxisList.length;i++){
        tempFilter.filterAxisList.push(new FilterAxis(selFilter.filterAxisList[i].axis, selFilter.filterAxisList[i].extent));
    }
    addFilter(tempFilter);
});

/**
 * Updates the order of the filters
*/
function updateOrder() {
    filterOrder = $( "#filters" ).sortable('toArray');
}

/**
 * this init the wrapper of the filterboxes. So the boxes can be changed by drag and drop (jquery ui library) 
*/
$( "#filters" ).sortable({
    stop: function(event, ui) {
        /*var changedList = this.id;
         var order = $(this).sortable('toArray');
         var positions = order.join(';');
         var test = 0;
         updateOrder();*/
        updateFilterLines();
        callbackMethod();
    }
});
$( "#filters" ).disableSelection();

/**
 * This method is called, when the a filterobject is changed (brush, order-change) 
 * It changes the attribute "lines" of all filters. The dataset is splitted into the correct filters objects
*/
function updateFilterLines() {
    updateOrder();
    for(var i = 0; i < filterList.length; i++) {
        filterList[i].lines = [];
    }
    for(var k = 0; k < _DATASET.length; k++) {
        for(var j = filterOrder.length-1; j >=0 ; j--) {
            var tempFilter = getFilterById(filterOrder[j]);
            var isDrawWithFilter = true;
            for(var i = 0; i < tempFilter.filterAxisList.length; i++) {
                var tempFilterAxis = tempFilter.filterAxisList[i];
                if(tempFilterAxis.extent[0] == tempFilterAxis.extent[1]) continue;
                var attr = tempFilterAxis.axis.attribute;
                if(tempFilterAxis.axis.isOrdinalAxis) {
                    if(!(tempFilterAxis.axis.axisScale(_DATASET[k][tempFilterAxis.axis.timeIndex][attr]) >= tempFilterAxis.extent[0]
                        && tempFilterAxis.axis.axisScale(_DATASET[k][tempFilterAxis.axis.timeIndex][attr]) <= tempFilterAxis.extent[1])) {
                        isDrawWithFilter = false;
                        break;
                    }
                } else {
                    if(!(_DATASET[k][tempFilterAxis.axis.timeIndex][attr] >= tempFilterAxis.extent[0]
                        && _DATASET[k][tempFilterAxis.axis.timeIndex][attr] <= tempFilterAxis.extent[1])) {
                        isDrawWithFilter = false;
                        break;
                    }
                }
            }

            if(isDrawWithFilter) {
                tempFilter.lines.push(_DATASET[k]);
                break;
            }
        }
    }
    callbackMethod();
}

/**
 * This method is called, when the selectedFilterId is changed.
 * All svg-elements and the brush of all axis is changed, to fit the active filter.
*/
function setAllBrushesToSelectedFilter(selectedFilterId) {
    for(var i = 0; i < _AXES.length; i++) {
        _AXES[i].brush.clear() ;
        _AXES[i].brush(_AXES[i].brushElement);
    }

    var selectedFilter = getFilterById(selectedFilterId);
    for(var i = 0; i < selectedFilter.filterAxisList.length; i++) {
        var tempaxis = selectedFilter.filterAxisList[i].axis;
        if(selectedFilter.filterAxisList[i].extent[0] == Number.NEGATIVE_INFINITY
            || selectedFilter.filterAxisList[i].extent[1] == Number.POSITIVE_INFINITY) {
            selectedFilter.filterAxisList[i].axis.brush.clear();
            selectedFilter.filterAxisList[i].axis.brush(selectedFilter.filterAxisList[i].axis.brushElement);
        } else {
            selectedFilter.filterAxisList[i].axis.brush.extent(selectedFilter.filterAxisList[i].extent);
            selectedFilter.filterAxisList[i].axis.brush(selectedFilter.filterAxisList[i].axis.brushElement);
        }
    }
    callbackMethod();

}

/**
 * If an axis is removed, all filterAxis also has to be removed.
 * This method removes all "filterAxis" from all filter objects.
*/
function removeAxisFromAllFilterAxisLists(filterAxis) {
    for(var i = 0; i < filterList.length; i++) {
        for(var j = 0; j < filterList[i].filterAxisList.length; j++) {
            if(filterList[i].filterAxisList[j].axis == filterAxis) {
                filterList[i].filterAxisList.splice(j, 1);
            }
        }
    }
    updateFilterLines();
}



