
var ordinalTypes = ["bending1", "bending2", "cycling", "lying", "sitting", "standing", "walking"];
var attributeExtents = {
    "avg_rss12": undefined,
    "var_rss12": undefined,
    "avg_rss13" : undefined,
    "var_rss13"   : undefined,
    "avg_rss23" : undefined,
    "var_rss23": undefined
}
/**
 * This method returns a new axisScale for the specific time
 * attributte: attributte name of the data
 * time: specific timestamp 
 * isOrdinal: true if the data are ordinal values
*/
function createAxisScale(attribute, time, isOrdinal) {
    if(isOrdinal) {
        var typenRange = [];
        for ( var i = 0; i < ordinalTypes.length; i++) {
            typenRange.push((margin + perspectiveScale(time) + i/(ordinalTypes.length-1) * (parallelPlotHeight- 2 * margin - 2*perspectiveScale(time))));
        }
        return d3.scale.ordinal()
            .domain(ordinalTypes)
            .range(typenRange);
    } else {
        if(attributeExtents[attribute] == undefined) {
            //extent bestimmung über alle zeitindexe
            var extents = [];
            for (var i = 0; i < _DATASET[0].length - 1; i++) {
                var tmp = d3.extent(_DATASET, function (d) {
                    return d[i][attribute];
                });
                extents.push(tmp[0]);
                extents.push(tmp[1]);
            }
            attributeExtents[attribute] = d3.extent(extents);
        }

        return d3.scale.linear()
            .domain(attributeExtents[attribute])
            .range([parallelPlotHeight-perspectiveScale(time)- margin, margin + perspectiveScale(time)]);
    }
}

/**
 * Tick constructor
 * data: datavalue
 * yvalue: y-coordinate on the axis
*/
function Tick(data, yValue) {
    this.data = data;
    this.y = yValue;
    this.yP = undefined;
}

/**
 * This method returns the value clmaped to the min max intevall
 * num: value
 * min: minimum value
 * max: maximum value
*/
function clamp(num, min, max) {
    return num <= min ? min : num >= max ? max : num;
}

/**
* This method returns the exact time in ms with the given timeIndex
*/
function calcTime(timeIndex) {
    return timeIndex * timeInterval;
}

/**
* This method returns the axis given by the id
*/
function getAxisById(id) {
    for (var i = 0; i < _AXES.length; i++) {
        if(+_AXES[i].id === +id) return _AXES[i];
    }
    return undefined;
}

/**
* This method returns the new postion calculated with the actual zoom and shift
*/
function rasterize(position) {
    return position * zoom + shift;
}

/**
* Updates the current status from the interaction buttons
*/
function updateGuiButtonsActions() {
    if(mouseMode === "select") {
        onClickSelectTool();
    } else if(mouseMode === "move") {
        onClickMoveTool();
    } else {
        onClickTimeTool();
    }
}