/**
 * This file stores all information for the interactions with the plot.
*/

var startDragX = 0;
var startDragY = 0;
var startTimeIndex = 0;

//Axis-Pos
{
    function draggedPosition() {
        var tempaxis = getAxisById(d3.select(this).attr('axesId'));
        updateAxisPositionWithoutZoom(tempaxis, d3.event.x);
        renderAxisWithSiblings(tempaxis);
        //renderAllAxes();
    }
    var dragPosition = d3.behavior.drag().on("drag", draggedPosition);
}

//Time
{
    function draggedTime() {
        var tempaxis = getAxisById(d3.select(this).attr('axesId'));
        var tempPercent = (d3.event.y - startDragY) / parallelPlotHeight;
        updateTimeAxis(tempaxis, Math.round(startTimeIndex + (timeScaleIndex[1] - timeScaleIndex[0]) * tempPercent));
        renderAxisWithSiblings(tempaxis);
    }
    var dragTime = d3.behavior.drag()
        .origin(function() {
            var tempaxis = getAxisById(d3.select(this).attr('axesId'));
            startTimeIndex = tempaxis.timeIndex;
            startDragX = d3.event.x;
            startDragY = d3.event.y;
            return {
                x: d3.event.x,
                y: d3.event.y
            };
        }).on("drag", draggedTime);
}

//Time-Area
{
    var startTimeIndexPrev = 0;
    function draggedTimeArea() {
        var tempaxis = getAxisById(d3.select(this).attr('axesId'));
        var tempPercent = (d3.event.y - startDragY) / parallelPlotHeight;
        updateTimeAxis(tempaxis, Math.round(startTimeIndex + (timeScaleIndex[1] - timeScaleIndex[0]) * tempPercent));
        renderAxisWithSiblings(tempaxis);
        updateTimeAxis(tempaxis.prevAxis, Math.round(startTimeIndex + (timeScaleIndex[1] - timeScaleIndex[0]) * tempPercent));
        renderAxisWithSiblings(tempaxis.prevAxis);
    }
    var dragTimeArea = d3.behavior.drag()
        .origin(function() {
            var tempaxis = getAxisById(d3.select(this).attr('axesId'));
            startTimeIndex = tempaxis.timeIndex;
            startTimeIndexPrev = tempaxis.prevAxis.timeIndex;
            startDragX = d3.event.x;
            startDragY = d3.event.y;
            return {
                x: d3.event.x,
                y: d3.event.y
            };
        }).on("drag", draggedTimeArea);
}

//Drag
{
    var startShift = 0;
    function draggedShift() {
        shift = shift + d3.event.x - startShift;
        startShift = d3.event.x;
        updateAllAxisPositions();
        renderAllAxes();
    }

    var dragShift = d3.behavior.drag()
        .origin(function() {
            startShift = d3.event.x;
            return {
                x: d3.event.x,
                y: d3.event.y
            };
        }).on("drag", draggedShift);
}

//Brush
{
    function brush(){
        for(var i=0;i<_AXES.length;i++){
            var extent= _AXES[i].brush.extent();
            filters.setExtentForAxis(_AXES[i], extent, _AXES[i].brush.empty());
        }
        filters.updateFilterLines();
        renderAllAxes();
    }
}

//Für Zoom das Mausrad abfragen
if(svgFrame[0][0].addEventListener) {
    svgFrame[0][0].addEventListener("mousewheel", MouseWheelHandler, false);
    svgFrame[0][0].addEventListener("DOMMouseScroll", MouseWheelHandler, false);
}

function MouseWheelHandler(e) {
    // cross-browser wheel delta
    var e = window.event || e; // old IE support
    var delta = Math.max(-1, Math.min(1, (e.wheelDelta || -e.detail)));
    if(mouseMode === "move") {
        var tBefore = zoom;
        zoom = zoom + delta/1; //oder /10
        updateAllAxisPositions();
        renderAllAxes();
    } else if (mouseMode === "time"){
        for(var i = 0; i < _AXES.length; i++) {
            updateTimeAxis(_AXES[i], _AXES[i].timeIndex + delta * 10);
        }
        renderAllAxes();
    }
}