/**
 * Implementation of the scatterplot
*/
var marginScatterPlot = {top: 10, right: 50, bottom: 50, left: 50};

var scatterplotList = [];
var scatterplotCounter = 0;
var scatterPlotWidth=400;
var scatterPlotHeight=360;

function Scatterplot(attribute_x, attribute_y, timeIndex_x, timeIndex_y) {
    this.attribute_x = attribute_x;
    this.attribute_y = attribute_y;
    this.timeIndex_x = timeIndex_x;
    this.timeIndex_y = timeIndex_y;
    this.id = scatterplotCounter;
    var tempDiv = d3.select("#overflowing").append("div").attr("class", "container").attr("id", "scatterplot-" + this.id).attr("style", "width:"+scatterPlotWidth+"px; height:"+scatterPlotHeight+"px");
    scatterplotCounter++;
    $( "#scatterplot-" + this.id ).resizable({
        resize: function(event, ui) {
            var tempDivWidth = $("#" + ui.element.attr("id")).width();
            var tempDivHeight = $("#" + ui.element.attr("id")).height();
            $("#scatterplot-svg-" + ui.element.attr("id").split("-")[1]).attr("width", tempDivWidth + "px");
            $("#scatterplot-svg-" + ui.element.attr("id").split("-")[1]).attr("height", tempDivHeight + "px");
            renderAllScatterplots();
        }
    }).draggable({ scroll: true, containment: "parent" });
    
    this.svg = tempDiv.append("svg").attr("id", "scatterplot-svg-" + this.id).attr("class", "scatterplot").attr("width", scatterPlotWidth+"px").attr("height", scatterPlotHeight+"px");
    
    this.xScale = d3.scale.linear().range([marginScatterPlot.left, this.svg[0][0].clientWidth - marginScatterPlot.right])
                    .domain(attributeExtents[this.attribute_x]);
    this.yScale = d3.scale.linear().range([this.svg[0][0].clientHeight  - marginScatterPlot.bottom, marginScatterPlot.top])
                    .domain(attributeExtents[this.attribute_y]);
    
    this.xAxis = d3.svg.axis().scale(this.xScale).orient("bottom");
    this.yAxis = d3.svg.axis().scale(this.yScale).orient("left");
    
    this.svg_xAxis = this.svg.append("g").attr("class", "axis xAxis");
    this.svg_xAxis.attr("transform", "translate(" + 0 + "," + (this.svg[0][0].clientHeight - marginScatterPlot.bottom) + ")").call(this.xAxis);
    this.textX= this.svg_xAxis.append("text")
                      .attr("class","axisLabel")
                      .attr("x", this.svg[0][0].clientWidth/2)
                      .attr("y", 35)
                      .text(this.attribute_x+" ("+ (this.timeIndex_x*250)+"ms)");
    this.svg_yAxis = this.svg.append("g").attr("class", "axis yAxis").attr("transform", "translate(" + marginScatterPlot.left + ",0)").call(this.yAxis);     
    this.textY= this.svg_yAxis.append("text")
                      .attr("class","axisLabel")
                      .attr("transform", "rotate(-90)")
                      .attr("x", -this.svg[0][0].clientHeight/2)
                      .attr("y", -35)
                      .text(this.attribute_y+" ("+ (this.timeIndex_y*250)+"ms)");
    this.circles = this.svg.append("g").attr("id", "circles-" + this.id);
}

function renderAllScatterplots() {
    for(var i = 0; i < scatterplotList.length; i++) {
        renderScatterplot(scatterplotList[i]);
    }
}

function renderScatterplot(scatterplot) {
    scatterplot.xScale.range([marginScatterPlot.left, scatterplot.svg[0][0].clientWidth - marginScatterPlot.right]);
    scatterplot.yScale.range([scatterplot.svg[0][0].clientHeight  - marginScatterPlot.bottom, marginScatterPlot.top]);

    scatterplot.svg_xAxis.attr("transform", "translate(" + 0 + "," + (scatterplot.svg[0][0].clientHeight - marginScatterPlot.bottom) + ")").call(scatterplot.xAxis);
    scatterplot.textX.attr("x", scatterplot.svg[0][0].clientWidth/2);
    scatterplot.svg_yAxis.attr("transform", "translate(" + marginScatterPlot.left + ",0)").call(scatterplot.yAxis);
    scatterplot.textY.attr("x", -scatterplot.svg[0][0].clientHeight/2);

    var tempGroup = scatterplot.circles.selectAll("g").data(filters.getFilterOrderIds());
    
    tempGroup.enter().append("g").attr("class", function(d, i) {
        return "grouping scatterplotid-" + scatterplot.id + "-circle-group-" + i; 
    });
    
    for(var i = 0; i < filters.getFilterOrderIds().length; i++) {
        var tempFilter = filters.getFilterById(filters.getFilterOrderIds()[i]);
        var tempFilterGroup = d3.selectAll(".scatterplotid-" + scatterplot.id + "-circle-group-" + i);
        var dots = tempFilterGroup.selectAll(".dot").data(tempFilter.lines);
        
        var xMap = function(d) { 
            return scatterplot.xScale(d[scatterplot.timeIndex_x][scatterplot.attribute_x]);};
        var yMap = function(d) { 
            return scatterplot.yScale(d[scatterplot.timeIndex_y][scatterplot.attribute_y]);};
        
        dots.enter().append("circle");
        var tempColor = tempFilter.color;
        dots.attr("r", 3.5)
          .attr("cx", xMap)
          .attr("cy", yMap)
          .attr("class", function(d) {
            return "linehover dot dot-" + d[scatterplot.timeIndex_x].index;
          })
          .style("fill","rgba(" + parseInt(tempColor.substring(1,3),16)+','+parseInt(tempColor.substring(3,5),16)+','+parseInt(tempColor.substring(5,7),16)+','+tempFilter.opacity / 100 +')');
        
        dots.exit().remove();
    } 
    tempGroup.exit().remove();
}

function addScatterPlotFromGUI(rightAxisID){
     var rightAxis=getAxisById(rightAxisID);
    if(rightAxis.isOrdinalAxis || rightAxis.prevAxis.isOrdinalAxis) {
        alert("One of the axes is ORDINAL! Please use a space without an ordinal axis."); 
        return;
    }
    addScatterPlot(rightAxis.prevAxis.attribute,rightAxis.attribute,rightAxis.prevAxis.timeIndex,rightAxis.timeIndex);
}

function addScatterPlot(attributeX,attributeY,timeIndexX,timeIndexY) {
    $("#other-plots").attr("style", "height:"+(scatterPlotHeight+340)+"px");
    scatterplotList.push(new Scatterplot(attributeX, attributeY, timeIndexX, timeIndexY));
    renderAllScatterplots();
    setTimeout(function() {
        updatePlotDimensions();
    },100);
}
