/**
 * Implementation of the time series
*/
var marginTimeSeries = {top: 10, right: 50, bottom: 50, left: 50};

var timeSeriesList = [];
var timeSeriesCounter = 0;
var timeSeriesWidth=1000;
var timeSeriesHeight=360;

function TimeSeries(attribute) {
    this.attribute= attribute;
    this.id = timeSeriesCounter;
    var tempDiv = d3.select("#overflowing").append("div").attr("class", "container").attr("id", "timeSeries-" + this.id).attr("style", "width:"+timeSeriesWidth+"px; height:"+timeSeriesHeight+"px");
    timeSeriesCounter++;
    $( "#timeSeries-" + this.id ).resizable({
        resize: function(event, ui) {
            var tempDivWidth = $("#" + ui.element.attr("id")).width();
            var tempDivHeight = $("#" + ui.element.attr("id")).height();
            $("#timeSeries-svg-" + ui.element.attr("id").split("-")[1]).attr("width", tempDivWidth + "px");
            $("#timeSeries-svg-" + ui.element.attr("id").split("-")[1]).attr("height", tempDivHeight + "px");
            renderAlltimeSeriess();
        }
    }).draggable({ scroll: true, containment: "parent" });
    
    this.svg = tempDiv.append("svg").attr("id", "timeSeries-svg-" + this.id).attr("class", "timeSeries").attr("width", timeSeriesWidth+"px").attr("height", timeSeriesHeight+"px");
    
    this.xScale = d3.scale.linear().range([marginTimeSeries.left, this.svg[0][0].clientWidth - marginTimeSeries.right])
                    .domain(timeScale);
    this.yScale = d3.scale.linear().range([this.svg[0][0].clientHeight  - marginTimeSeries.bottom, marginTimeSeries.top])
                    .domain(attributeExtents[this.attribute]);
    
    this.xAxis = d3.svg.axis().scale(this.xScale).orient("bottom");
    this.yAxis = d3.svg.axis().scale(this.yScale).orient("left");
    
    this.svg_xAxis = this.svg.append("g").attr("class", "axis xAxis");
    this.svg_xAxis.attr("transform", "translate(" + 0 + "," + (this.svg[0][0].clientHeight - marginTimeSeries.bottom) + ")").call(this.xAxis);
    this.textX= this.svg_xAxis.append("text")
                      .attr("class","axisLabel")
                      .attr("x", this.svg[0][0].clientWidth/2)
                      .attr("y", 35)
                      .text("time in ms");
    this.svg_yAxis = this.svg.append("g").attr("class", "axis yAxis").attr("transform", "translate(" + marginTimeSeries.left + ",0)").call(this.yAxis);     
    this.textY= this.svg_yAxis.append("text")
                      .attr("class","axisLabel")
                      .attr("transform", "rotate(-90)")
                      .attr("x", -this.svg[0][0].clientHeight/2)
                      .attr("y", -35)
                      .text(this.attribute);
    this.lines = this.svg.append("g").attr("id", "lines");
}

function renderAlltimeSeriess() {
    for(var i = 0; i < timeSeriesList.length; i++) {
        rendertimeSeries(timeSeriesList[i]);
    }
}

function rendertimeSeries(timeSeries) {
    
    timeSeries.xScale.range([marginTimeSeries.left, timeSeries.svg[0][0].clientWidth - marginTimeSeries.right]).domain(timeScale);;
    timeSeries.yScale.range([timeSeries.svg[0][0].clientHeight  - marginTimeSeries.bottom, marginTimeSeries.top]);

    timeSeries.svg_xAxis.attr("transform", "translate(" + 0 + "," + (timeSeries.svg[0][0].clientHeight - marginTimeSeries.bottom) + ")").call(timeSeries.xAxis);
    timeSeries.textX.attr("x", timeSeries.svg[0][0].clientWidth/2);
    timeSeries.svg_yAxis.attr("transform", "translate(" + marginTimeSeries.left + ",0)").call(timeSeries.yAxis);
    timeSeries.textY.attr("x", -timeSeries.svg[0][0].clientHeight/2);
    
    var line = d3.svg.line()
    .x(function(d) { return timeSeries.xScale(d.time); })
    .y(function(d) { return timeSeries.yScale(d[timeSeries.attribute]); });
    

    var tempGroup = timeSeries.lines.selectAll("g").data(filters.getFilterOrderIds());
    
    tempGroup.enter().append("g").attr("class", function(d, i) {
        return "timeSeriesid-" + timeSeries.id + "-path-group-" + i; 
    });
    
    for(var i = 0; i < filters.getFilterOrderIds().length; i++) {
        var tempFilter = filters.getFilterById(filters.getFilterOrderIds()[i]);
        var dataSetWithTimeIndex=[];
        for(var j = 0; j < tempFilter.lines.length; j++) {
                    var tempDataset = tempFilter.lines[j];
                    dataSetWithTimeIndex.push(tempFilter.lines[j].slice(timeScaleIndex[0], timeScaleIndex[1]+1));
        }
        var tempFilterGroup = d3.selectAll(".timeSeriesid-" + timeSeries.id + "-path-group-" + i);
        var path = tempFilterGroup.selectAll(".path").data(dataSetWithTimeIndex);  
        path.enter().append("path");
        var tempColor = tempFilter.color;
        path.attr("d",function(d){
            return line(d);})
       .attr("class", function(d) {
            return "linehover path path-" + d[0].index;
          })
        .style("stroke","rgba(" + parseInt(tempColor.substring(1,3),16)+','+parseInt(tempColor.substring(3,5),16)+','+parseInt(tempColor.substring(5,7),16)+','+tempFilter.opacity / 100 +')');
         path.exit().remove();
    } 
}


function addTimeSeries(axisId){
    if(getAxisById(axisId).isOrdinalAxis) {
        alert("One of the axes is ORDINAL! Please use a nonordinal axis."); 
        return;
    }
    $("#other-plots").attr("style", "height:"+(timeSeriesHeight+340)+"px");
    timeSeriesList.push(new TimeSeries(getAxisById(axisId).attribute));
    renderAlltimeSeriess();
        setTimeout(function() {
        updatePlotDimensions();
    },100);
}